<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use Infomaker\Everyware\Support\NewRelicLog;
use Twig\Environment;
use Twig\Loader\FilesystemLoader;
use Twig\Util\DeprecationCollector;

/**
 * Class DeprecationHandler
 * @package Infomaker\Everyware\Twig
 */
class DeprecationHandler
{
    /**
     * @var DeprecationCollector
     */
    private $collector;

    /**
     * @var array
     */
    private $registeredPaths = [];

    /**
     * @var Environment
     */
    private $environment;

    public function __construct(Environment $twig)
    {
        $this->environment = $twig;
        $this->collector = new DeprecationCollector($twig);
    }

    public function search(FilesystemLoader $loader): array
    {
        $paths = $this->getRegisteredPaths($loader);
        if( empty($paths) ) {
            return [];
        }
        $deprecations = array_map([$this->collector, 'collectDir'], $paths);

        return array_merge(...$deprecations);
    }

    public function searchAndLog(): void
    {
        $loader = $this->environment->getLoader();
        if ( $loader instanceof FilesystemLoader) {
            $deprecationWarnings = $this->search($loader);

            foreach ($deprecationWarnings as $warning) {
                NewRelicLog::error($warning);
            }
        }
    }

    public function listen(): void
    {
        set_error_handler(static function ($type, $msg) {
            if (E_USER_DEPRECATED === $type) {
                NewRelicLog::error($msg);
            }
        });
    }

    private function getRegisteredPaths(FilesystemLoader $loader): array
    {
        $paths = [[]];
        foreach ($loader->getNamespaces() as $namespace) {
            $paths[] = $this->registeredPaths[$namespace] = $loader->getPaths($namespace);
        }

        return array_merge(...$paths);
    }
}
