<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig\Extension;

use Infomaker\Everyware\Support\Device;
use Infomaker\Everyware\Support\Environment;
use Infomaker\Everyware\Twig\Binary\ContainsBinary;
use Twig\ExpressionParser;
use Twig\Extension\AbstractExtension;
use Twig\Extension\GlobalsInterface;
use Twig\Node\Expression\Binary\AndBinary;
use Twig\Node\Expression\Binary\OrBinary;
use Twig\Node\Expression\Unary\NotUnary;

/**
 * Class ProjectExtension
 * @package Infomaker\Everyware\Twig
 */
class ProjectExtension extends AbstractExtension implements GlobalsInterface
{

    /** @var array */
    private $filters = [];

    /** @var array */
    private $functions = [];

    /** @var array */
    private $globals = [];

    /** @var array */
    private $operators = [];

    /** @var array */
    private $tests = [];


    /**
     * @inheritDoc
     */
    public function getFilters(): array
    {
        return array_replace($this->filters, $this->getResources(new Filters()) ?? parent::getFilters());
    }

    /**
     * @inheritDoc
     */
    public function getFunctions(): array
    {
        return array_replace($this->functions, $this->getResources(new Functions()) ?? parent::getFunctions());
    }

    /**
     * @inheritDoc
     */
    public function getGlobals(): array
    {
        return [
            'device' => Device::currentDevice(),
            'in_production' => Environment::isProd()
        ];
    }

    public function getOperators(): array
    {
        return [
            [ // unary operator.
                '!' => ['precedence' => 50, 'class' => NotUnary::class],
            ],
            [ // binary operator
                '||' => ['precedence' => 10, 'class' => OrBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
                '&&' => ['precedence' => 15, 'class' => AndBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
                'contains' => ['precedence' => 20,'class' => ContainsBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT ]
            ]
        ];
    }

    /**
     * @inheritDoc
     */
    public function getTests(): array
    {
        return array_replace($this->tests, $this->getResources(new Tests()) ?? parent::getTests());
    }

    private function getResources($resources)
    {
        if ($resources instanceof ExtensionResourceInterface) {
            return $resources->getResourceList();
        }

        return null;
    }

}
